@@ -31,7 +31,7 @@ use Exception::Class (
 	'Travel::Routing::DE::EFA::Exception::Ambiguous' => {
 		isa         => 'Travel::Routing::DE::EFA::Exception',
 		description => 'ambiguous input',
-		fields      => [ 'post_key', 'possibilities' ],
+		fields      => [ 'post_key', 'post_value', 'possibilities' ],
 	},
 	'Travel::Routing::DE::EFA::Exception::Other' => {
 		isa         => 'Travel::Routing::DE::EFA::Exception',
@@ -40,7 +40,7 @@ use Exception::Class (
 	},
 );
 
-our $VERSION = '2.07';
+our $VERSION = '2.09';
 
 sub set_time {
 	my ( $self, %conf ) = @_;
@@ -687,8 +687,10 @@ sub check_ambiguous_xml {
 	my $xp_place = XML::LibXML::XPathExpression->new('./itdOdvPlace');
 	my $xp_name  = XML::LibXML::XPathExpression->new('./itdOdvName');
 
-	my $xp_place_elem = XML::LibXML::XPathExpression->new('./odvPlaceElem');
-	my $xp_name_elem  = XML::LibXML::XPathExpression->new('./odvNameElem');
+	my $xp_place_elem  = XML::LibXML::XPathExpression->new('./odvPlaceElem');
+	my $xp_place_input = XML::LibXML::XPathExpression->new('./odvPlaceInput');
+	my $xp_name_elem   = XML::LibXML::XPathExpression->new('./odvNameElem');
+	my $xp_name_input  = XML::LibXML::XPathExpression->new('./odvNameInput');
 
 	my $e_place = ( $tree->findnodes($xp_place) )[0];
 	my $e_name  = ( $tree->findnodes($xp_name) )[0];
@@ -703,7 +705,11 @@ sub check_ambiguous_xml {
 
 	if ( $s_place eq 'list' ) {
 		Travel::Routing::DE::EFA::Exception::Ambiguous->throw(
-			post_key      => 'place',
+			post_key   => 'place',
+			post_value => decode(
+				'UTF-8',
+				( $e_place->findnodes($xp_place_input) )[0]->textContent
+			),
 			possibilities => join( q{ | },
 				map { decode( 'UTF-8', $_->textContent ) }
 				  @{ $e_place->findnodes($xp_place_elem) } )
@@ -711,7 +717,10 @@ sub check_ambiguous_xml {
 	}
 	if ( $s_name eq 'list' ) {
 		Travel::Routing::DE::EFA::Exception::Ambiguous->throw(
-			post_key      => 'name',
+			post_key   => 'name',
+			post_value => decode(
+				'UTF-8', ( $e_name->findnodes($xp_name_input) )[0]->textContent
+			),
 			possibilities => join( q{ | },
 				map { decode( 'UTF-8', $_->textContent ) }
 				  @{ $e_name->findnodes($xp_name_elem) } )
@@ -745,39 +754,61 @@ sub routes {
 # static
 sub get_efa_urls {
 	return (
-		[
-			'http://efa.ivb.at/ivb/XSLT_TRIP_REQUEST2',
-			'Innsbrucker Verkehsbetriebe'
-		],
-		[
-			'http://efa.svv-info.at/sbs/XSLT_TRIP_REQUEST2',
-			'Salzburger Verkehrsverbund'
-		],
-		[
-			'http://efa.vor.at/wvb/XSLT_TRIP_REQUEST2',
-			'Verkehrsverbund Ost-Region'
-		],
-		[
-			'http://efaneu.vmobil.at/vvv/XSLT_TRIP_REQUEST2',
-			'Vorarlberger Verkehrsverbund'
-		],
-		[
-			'http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2',
-			'Verkehsverbund Steiermark'
-		],
-		[ 'http://www.linzag.at/static/XSLT_TRIP_REQUEST2', 'Linz AG' ],
-		[
-			'http://212.114.197.7/vgnExt_oeffi/XML_TRIP_REQUEST2',
-			'Verkehrsverbund Grossraum Nuernberb'
-		],
-		[
-			'http://efa.vrr.de/vrr/XSLT_TRIP_REQUEST2',
-			'Verkehrsverbund Rhein-Ruhr'
-		],
-		[
-			'http://www2.vvs.de/vvs/XSLT_TRIP_REQUEST2',
-			'Verkehrsverbund Stuttgart'
-		],
+		{
+			url       => 'http://www.ding.eu/ding3/XSLT_TRIP_REQUEST2',
+			name      => 'Donau-Iller Nahverkehrsverbund',
+			shortname => 'DING',
+		},
+		{
+			url       => 'http://efa.ivb.at/ivb/XSLT_TRIP_REQUEST2',
+			name      => 'Innsbrucker Verkehrsbetriebe',
+			shortname => 'IVB',
+		},
+		{
+			url       => 'http://efa.svv-info.at/sbs/XSLT_TRIP_REQUEST2',
+			name      => 'Salzburger Verkehrsverbund',
+			shortname => 'SVV',
+		},
+		{
+			url       => 'http://efa.vor.at/wvb/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Ost-Region',
+			shortname => 'VOR',
+		},
+		{
+			url       => 'http://efaneu.vmobil.at/vvv/XSLT_TRIP_REQUEST2',
+			name      => 'Vorarlberger Verkehrsverbund',
+			shortname => 'VVV',
+		},
+		{
+			url  => 'http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2',
+			name => 'Verkehrsverbund Steiermark',
+			shortname => 'Verbundlinie',
+		},
+		{
+			url       => 'http://www.linzag.at/static/XSLT_TRIP_REQUEST2',
+			name      => 'Linz AG',
+			shortname => 'LinzAG',
+		},
+		{
+			url       => 'http://212.114.197.7/vgnExt_oeffi/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Grossraum Nuernberg',
+			shortname => 'VGN',
+		},
+		{
+			url       => 'http://efa.vrr.de/vrr/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Rhein-Ruhr',
+			shortname => 'VRR',
+		},
+		{
+			url       => 'http://app.vrr.de/standard/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Rhein-Ruhr (alternative)',
+			shortname => 'VRR2',
+		},
+		{
+			url       => 'http://www2.vvs.de/vvs/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Stuttgart',
+			shortname => 'VVS',
+		},
 	);
 }
 
@@ -802,18 +833,18 @@ Travel::Routing::DE::EFA - unofficial interface to EFA-based itinerary services
 	for my $route ( $efa->routes ) {
 		for my $part ( $route->parts ) {
 			printf(
-				"%s at %s -> %s at %s, via %s to %s",
+				"%s at %s -> %s at %s, via %s to %s\n",
 				$part->departure_time, $part->departure_stop,
 				$part->arrival_time,   $part->arrival_stop,
 				$part->train_line,     $part->train_destination,
 			);
 		}
-		print "\n\n";
+		print "\n";
 	}
 
 =head1 VERSION
 
-version 2.07
+version 2.09
 
 =head1 DESCRIPTION
 
@@ -849,7 +880,7 @@ is not completely supported yet and may not work at all.
 
 =over
 
-=item * L<http://efa.ivb.at/ivb/XSLT_TRIP_REQUEST2> (Innsbrucker Verkehsbetriebe)
+=item * L<http://efa.ivb.at/ivb/XSLT_TRIP_REQUEST2> (Innsbrucker Verkehrsbetriebe)
 
 =item * L<http://efa.svv-info.at/sbs/XSLT_TRIP_REQUEST2> (Salzburger Verkehrsverbund)
 
@@ -857,7 +888,7 @@ is not completely supported yet and may not work at all.
 
 =item * L<http://efaneu.vmobil.at/vvv/XSLT_TRIP_REQUEST2> (Vorarlberger Verkehrsverbund)
 
-=item * L<http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2> (Verkehsverbund Steiermark) B<[!]>
+=item * L<http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2> (Verkehrsverbund Steiermark) B<[!]>
 
 =item * L<http://www.linzag.at/static/XSLT_TRIP_REQUEST2> (Linz AG) B<[!]>
 
@@ -871,6 +902,8 @@ is not completely supported yet and may not work at all.
 
 =item * L<http://efa.vrr.de/vrr/XSLT_TRIP_REQUEST2> (Verkehrsverbund Rhein-Ruhr)
 
+=item * L<http://app.vrr.de/standard/XML_TRIP_REQUEST2> (Verkehrsverbund Rhein-Ruhr with support for B<--full-route>)
+
 =item * L<http://www2.vvs.de/vvs/XSLT_TRIP_REQUEST2> (Verkehrsverbund Stuttgart)
 
 =back
@@ -905,7 +938,7 @@ Journey start time.  Default: now
 
 =item B<date> => I<DD.MM.>[I<YYYY>]
 
-Journey date.  Default: tdoay
+Journey date.  Default: today
 
 =item B<exclude> => \@exclude
 
@@ -1000,11 +1033,20 @@ The following methods act like the arguments to B<new>. See there.
 
 =over
 
-=item Travel::Status::DE::VRR::get_efa_urls()
+=item Travel::Routing::DE::EFA::get_efa_urls()
 
-Returns a list of known EFA entry points. Each list element is a reference to
-an array consisting of two strings. The first one is the URL (as passed to
-B<efa_url>), the second describes the entity to which this URL belongs.
+Returns a list of known EFA entry points. Each list element is a hashref with
+the following elements.
+
+=over
+
+=item B<url>: service URL as passed to B<efa_url>
+
+=item B<name>: Name of the entity operating this service
+
+=item B<shortname>: Short name of the entity
+
+=back
 
 =back
 
@@ -1041,7 +1083,7 @@ None known.
 
 =head1 AUTHOR
 
-Copyright (C) 2009-2014 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2009-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
